use tracing::error;

mod service;

#[tokio::main]
async fn main() {
    let file_appender = tracing_appender::rolling::hourly(
        &std::env::current_exe()
            .unwrap()
            .parent()
            .unwrap()
            .display()
            .to_string(),
        "prefix.log",
    );
    let (non_blocking, _guard) = tracing_appender::non_blocking(file_appender);

    tracing_subscriber::fmt()
        .with_max_level(tracing::Level::DEBUG)
        .with_target(false)
        .with_writer(non_blocking)
        // .with_timer(timer)
        .pretty()
        .init();

    if let Err(e) = service::run_service().await {
        error!("{e:?}");
    }

    println!("end of running");
}
