use axum::{routing::get, Router};
use error_stack::Report;
use tokio::sync::mpsc::Receiver;

use super::ServiceError;

pub(super) async fn execute(mut stop_signal: Receiver<()>) -> Result<(), Report<ServiceError>> {
    // build our application with a single route
    let app = Router::new().route("/", get(|| async { "Hello, World!" }));

    // run it with hyper on localhost:3000
    axum::Server::bind(&"0.0.0.0:3000".parse().unwrap())
        .serve(app.into_make_service())
        .with_graceful_shutdown(async {
            stop_signal.recv().await;
        })
        .await
        .unwrap();

    Ok(())
}
