use std::ffi::OsString;

use error_stack::{IntoReport, Report, ResultExt};
use windows_service::{
    service::{ServiceAccess, ServiceErrorControl, ServiceInfo, ServiceStartType, ServiceType},
    service_manager::{ServiceManager, ServiceManagerAccess},
};

use crate::service::ServiceError;

pub fn install() -> Result<(), Report<ServiceError>> {
    let manager_access = ServiceManagerAccess::CONNECT | ServiceManagerAccess::CREATE_SERVICE;
    let service_manager = ServiceManager::local_computer(None::<&str>, manager_access)
        .into_report()
        .change_context(ServiceError::Installing)?;

    // This example installs the service defined in `examples/ping_service.rs`.
    // In the real world code you would set the executable path to point to your own binary
    // that implements windows service.
    let service_binary_path = ::std::env::current_exe()
        .into_report()
        .change_context(ServiceError::Installing)?
        .with_file_name(format!("{}.exe", env!("CARGO_PKG_NAME")));

    let service_info = ServiceInfo {
        name: OsString::from(env!("CARGO_PKG_NAME")),
        display_name: OsString::from("DaRL"),
        service_type: ServiceType::OWN_PROCESS,
        start_type: ServiceStartType::AutoStart,
        error_control: ServiceErrorControl::Normal,
        executable_path: service_binary_path,
        launch_arguments: vec!["--serviced".into()],
        dependencies: vec![],
        account_name: None, // run as System
        account_password: None,
    };
    let service = service_manager
        .create_service(&service_info, ServiceAccess::CHANGE_CONFIG)
        .into_report()
        .change_context(ServiceError::Installing)?;

    service
        .set_description("DaRL - Data and Runtime Logger service")
        .into_report()
        .change_context(ServiceError::Installing)?;
    Ok(())
}
