use std::{thread, time::Duration};

use error_stack::{IntoReport, Report, ResultExt};
use windows_service::{
    service::{ServiceAccess, ServiceState},
    service_manager::{ServiceManager, ServiceManagerAccess},
};

use crate::service::ServiceError;

pub fn uninstall() -> Result<(), Report<ServiceError>> {
    let manager_access = ServiceManagerAccess::CONNECT;
    let service_manager = ServiceManager::local_computer(None::<&str>, manager_access)
        .into_report()
        .change_context(ServiceError::Uninstalling)?;

    let service_access = ServiceAccess::QUERY_STATUS | ServiceAccess::STOP | ServiceAccess::DELETE;
    let service = service_manager
        .open_service(env!("CARGO_PKG_NAME"), service_access)
        .into_report()
        .change_context(ServiceError::Uninstalling)?;

    let service_status = service
        .query_status()
        .into_report()
        .change_context(ServiceError::Uninstalling)?;
    if service_status.current_state != ServiceState::Stopped {
        service
            .stop()
            .into_report()
            .change_context(ServiceError::Uninstalling)?;
        // Wait for service to stop
        thread::sleep(Duration::from_secs(1));
    }

    service
        .delete()
        .into_report()
        .change_context(ServiceError::Uninstalling)?;

    Ok(())
}
